@@ -286,15 +286,12 @@
 ;; Dispatch for number representation
 
 (##define-macro (macro-number-dispatch num err fix big rat flo cpx)
-  `(cond ((##fixnum? ,num)                            ,fix)
-         ((##flonum? ,num)                            ,flo)
-         ((##subtyped? ,num)
-          (let ((##s (##subtype ,num)))
-            (cond ((##fx= ##s (macro-subtype-bignum)) ,big)
-                  ((##fx= ##s (macro-subtype-ratnum)) ,rat)
-                  ((##fx= ##s (macro-subtype-cpxnum)) ,cpx)
-                  (else                               ,err))))
-         (else                                        ,err)))
+  `(cond ((##fixnum? ,num) ,fix)
+         ((##flonum? ,num) ,flo)
+         ((##bignum? ,num) ,big)
+         ((##ratnum? ,num) ,rat)
+         ((##cpxnum? ,num) ,cpx)
+         (else             ,err)))
 
 ;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
@@ -444,15 +441,15 @@
 ;; slot 0 = numerator
 ;; slot 1 = denominator
 
+;;TODO: replace with ##ratnum-make
+
 (##define-macro (macro-ratnum-make num den)
   `(##subtype-set!
     (##vector ,num ,den)
     (macro-subtype-ratnum)))
 
 (##define-macro (macro-ratnum-numerator r)          `(macro-slot 0 ,r))
-(##define-macro (macro-ratnum-numerator-set! r x)   `(macro-slot 0 ,r ,x))
 (##define-macro (macro-ratnum-denominator r)        `(macro-slot 1 ,r))
-(##define-macro (macro-ratnum-denominator-set! r x) `(macro-slot 1 ,r ,x))
 
 ;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
@@ -462,15 +459,15 @@
 ;; slot 0 = real
 ;; slot 1 = imag
 
+;;TODO: replace with ##cpxnum-make
+
 (##define-macro (macro-cpxnum-make r i)
   `(##subtype-set!
     (##vector ,r ,i)
     (macro-subtype-cpxnum)))
 
 (##define-macro (macro-cpxnum-real c)        `(macro-slot 0 ,c))
-(##define-macro (macro-cpxnum-real-set! c x) `(macro-slot 0 ,c ,x))
 (##define-macro (macro-cpxnum-imag c)        `(macro-slot 1 ,c))
-(##define-macro (macro-cpxnum-imag-set! c x) `(macro-slot 1 ,c ,x))
 
 ;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
